BorrowSwapResult

Enum BorrowSwapResult 

pub enum BorrowSwapResult<'a, T: Sync + Send + 'static, E: Sync + Send + 'static> {
    BorrowSwapped(BorrowGuard<'a, T>),
    CellWasEmpty(E),
}
Result enum to indicate the outcomes of a borrow_swap operation.

Variants



BorrowSwapped(BorrowGuard<'a, T>)

Borrow swap succeeded and the returned guard should be used.



CellWasEmpty(E)

Cell was empty when the borrow swap operation was performed

Implementations

Source

impl<'a, T: Sync + Send + 'static, E: Sync + Send + 'static> BorrowSwapResult<'a, T, E>

pub fn unwrap(self) -> BorrowGuard<'a, T>

Unwraps the borrow guard

Panics

if CellWasEmpty

pub const fn is_ok(&self) -> bool

pub const fn is_err(&self) -> bool

pub fn unwrap_err(self) -> E

Unwraps the value intended as replacement as result of a failed operation.

Panics

if BorrowSwapped

pub fn map_err<X: Sync + Send + 'static>( self, f: impl FnOnce(E) -> X, ) -> BorrowSwapResult<'a, T, X>

Maps the type in the CellWasEmpty case. Does nothing for BorrowSwapped.

pub fn into_result(self) -> Result<BorrowGuard<'a, T>, E>

Transforms this into a core::Result type.

Errors

In case of CellWasEmpty