pub enum BorrowSwapResult<'a, T: Sync + Send + 'static, E: Sync + Send + 'static> {
BorrowSwapped(BorrowGuard<'a, T>),
CellWasEmpty(E),
}Expand description
Result enum to indicate the outcomes of a borrow_swap operation.
Variants
BorrowSwapped(BorrowGuard<'a, T>)
Borrow swap succeeded and the returned guard should be used.
CellWasEmpty(E)
Cell was empty when the borrow swap operation was performed
Implementations
Sourceimpl<'a, T: Sync + Send + 'static, E: Sync + Send + 'static> BorrowSwapResult<'a, T, E>
impl<'a, T: Sync + Send + 'static, E: Sync + Send + 'static> BorrowSwapResult<'a, T, E>
Sourcepub fn unwrap(self) -> BorrowGuard<'a, T>
pub fn unwrap(self) -> BorrowGuard<'a, T>
pub const fn is_ok(&self) -> bool
pub const fn is_err(&self) -> bool
Sourcepub fn unwrap_err(self) -> E
pub fn unwrap_err(self) -> E
Sourcepub fn map_err<X: Sync + Send + 'static>(
self,
f: impl FnOnce(E) -> X,
) -> BorrowSwapResult<'a, T, X>
pub fn map_err<X: Sync + Send + 'static>( self, f: impl FnOnce(E) -> X, ) -> BorrowSwapResult<'a, T, X>
Maps the type in the CellWasEmpty case. Does nothing for BorrowSwapped.
Sourcepub fn into_result(self) -> Result<BorrowGuard<'a, T>, E>
pub fn into_result(self) -> Result<BorrowGuard<'a, T>, E>